package info.silin.an10na.bind;

import info.silin.an10na.converter.IConverter;
import info.silin.an10na.observer.FieldObserver;
import info.silin.an10na.observer.IObserver;
import info.silin.an10na.observer.MethodObserver;
import info.silin.an10na.setter.FieldSetter;
import info.silin.an10na.setter.ISetter;
import info.silin.an10na.setter.MethodSetter;

/**
 * Binds the values of two fields together. The values are usually of different types and need to be
 * converted.The binding can be realized by methods, so a method on the source is called and the
 * return value is converted and passed to a method on the target.
 *
 * @author kostja
 */
public class ConvertingDataBinding<S, T> implements IDataBinding<S, T> {

    private final IObserver<S> observer;

    private final IConverter<S, T> converter;

    private final ISetter<T> targetSetter;

    public ConvertingDataBinding(final Object subject, final String sourceName, final boolean isSourceMethod,
                                 final Class<S> sourceKlazz, final Object target, final String targetName, final Class<T> targetKlazz, final boolean isTargetMethod, final IConverter<S, T> converter) {

        if (isSourceMethod) {
            observer = new MethodObserver<S>(subject, sourceName, sourceKlazz);
        } else {
            observer = new FieldObserver<S>(subject, sourceName, sourceKlazz);
        }
        if (isTargetMethod) {
            targetSetter = new MethodSetter<T>(target, targetName, targetKlazz);
        } else {
            targetSetter = new FieldSetter<T>(target, targetName, targetKlazz);
        }
        this.converter = converter;
    }

    public ConvertingDataBinding(final IObserver<S> observer, final IConverter<S, T> converter, final ISetter<T> setter) {
        this.observer = observer;
        this.converter = converter;
        this.targetSetter = setter;
    }

    @Override
    public void update() {
        targetSetter.set(converter.convert(observer.getCurrent()));
    }

    public IObserver<S> getObserver() {
        return observer;
    }

    public IConverter<S, T> getConverter() {
        return converter;
    }

    public ISetter<T> getTargetSetter() {
        return targetSetter;
    }
}
