package info.silin.an10na.bind;

import info.silin.an10na.observer.FieldObserver;
import info.silin.an10na.observer.IObserver;
import info.silin.an10na.observer.MethodObserver;
import info.silin.an10na.setter.FieldSetter;
import info.silin.an10na.setter.ISetter;
import info.silin.an10na.setter.MethodSetter;

/**
 * Binds the values of two fields of the same type together. The binding can be realized by
 * methods,
 * so a method on the source is called and the return value is passed to a method on the target.
 *
 * @author kostja
 */
public class DirectDataBinding<T> implements IDataBinding<T, T> {

    private final IObserver<T> observer;

    private final ISetter<T> targetSetter;

    public DirectDataBinding(final Object subject, final String sourceName,
                             final boolean sourceMethod, final Object target,
                             final String targetName, final Class<T> klazz, final boolean method) {

        if (sourceMethod) {
            observer = new MethodObserver<T>(subject, sourceName, klazz);
        } else {
            observer = new FieldObserver<T>(subject, sourceName, klazz);
        }
        if (method) {
            targetSetter = new MethodSetter<T>(target, targetName, klazz);
        } else {
            targetSetter = new FieldSetter<T>(target, targetName, klazz);
        }
    }

    public DirectDataBinding(final IObserver<T> observer, final ISetter<T> setter) {
        this.observer = observer;
        this.targetSetter = setter;
    }

    @Override
    public void update() {
        targetSetter.set(observer.getCurrent());
    }

    public IObserver<T> getObserver() {
        return observer;
    }

    public ISetter<T> getTargetSetter() {
        return targetSetter;
    }
}
