package info.silin.an10na.observer;

import info.silin.an10na.converter.IConverter;

/**
 * Keeps track of a field or method of an Object. Converts the value on return from <T> to <S>.
 * @author kostja
 */
public class ConvertingObserver<T, S> implements IObserver<T> {

    private final IObserver<S> observer;

    private final IConverter<S, T> converter;

    public ConvertingObserver(final Object subject, final String name, final boolean method, final Class<S> klazz,
                              final IConverter<S, T> converter) {
        if (method) {
            observer = new MethodObserver<S>(subject, name, klazz);
        } else {
            observer = new FieldObserver<S>(subject, name, klazz);
        }
        this.converter = converter;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = (prime * result)
                + ((converter == null) ? 0 : converter.hashCode());
        result = (prime * result)
                + ((observer == null) ? 0 : observer.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        @SuppressWarnings("unchecked") final
        ConvertingObserver<T, S> other = (ConvertingObserver<T, S>) obj;
        if (converter == null) {
            if (other.converter != null)
                return false;
        } else if (!converter.equals(other.converter))
            return false;
        if (observer == null) {
            if (other.observer != null)
                return false;
        } else if (!observer.equals(other.observer))
            return false;
        return true;
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#update()
    */
    @Override
    public void update() {
        observer.update();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#get()
    */
    @Override
    public T get() {
        return converter.convert(observer.get());
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getCurrent()
    */
    @Override
    public T getCurrent() {
        update();
        return get();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSourceName()
    */
    @Override
    public String getSourceName() {
        return observer.getSourceName();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSource()
    */
    @Override
    public Object getSource() {
        return observer.getSource();
    }
}
