package info.silin.an10na.observer;

import static org.fest.reflect.core.Reflection.field;

/**
 * Keeps track of a field on a source Object. Does not keep track automatically,
 *
 * @param <T> - the type of the observed field
 * @author kostja
 */
public class FieldObserver<T> implements IObserver<T> {

    private final Object source;
    private final String fieldName;
    private final Class<T> sourceClass;
    private T value;

    public FieldObserver(final Object source, final String name, final Class<T> sourceClass) {
        this.source = source;
        this.fieldName = name;
        this.sourceClass = sourceClass;

        value = field(fieldName).ofType(sourceClass).in(source).get();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#update()
    */
    @Override
    public void update() {
        value = field(fieldName).ofType(sourceClass).in(source).get();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#get()
    */
    @Override
    public T get() {
        return value;
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getCurrent()
    */
    @Override
    public T getCurrent() {
        update();
        return get();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSourceName()
    */
    @Override
    public String getSourceName() {
        return fieldName;
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSource()
    */
    @Override
    public Object getSource() {
        return source;
    }

    /*
    * (non-Javadoc)
    *
    * @see java.lang.Object#hashCode()
    */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = (prime * result)
                + ((fieldName == null) ? 0 : fieldName.hashCode());
        result = (prime * result) + ((source == null) ? 0 : source.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        @SuppressWarnings("unchecked") final
        FieldObserver<T> other = (FieldObserver<T>) obj;
        if (fieldName == null) {
            if (other.fieldName != null)
                return false;
        } else if (!fieldName.equals(other.fieldName))
            return false;
        if (source == null) {
            if (other.source != null)
                return false;
        } else if (!source.equals(other.source))
            return false;
        return true;
    }
}
