package info.silin.an10na.observer;

import static org.fest.reflect.core.Reflection.method;

/**
 * @author kostja
 */
//TODO : extend with parametrized methods
public class MethodObserver<T> implements IObserver<T> {

    private final Object subject;
    private final String methodName;
    private final Class<T> methodReturnClass;
    private T value;

    public MethodObserver(final Object subject, final String methodName, final Class<T> methodReturnClass) {
        this.subject = subject;
        this.methodName = methodName;
        this.methodReturnClass = methodReturnClass;

        value = method(methodName).withReturnType(methodReturnClass).in(subject).invoke();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#update()
    */
    @Override
    public void update() {
        value = method(methodName).withReturnType(methodReturnClass).in(subject).invoke();
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#get()
    */
    @Override
    public T get() {
        return value;
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getCurrent()
    */
    @Override
    public T getCurrent() {
        update();
        return get();
    }


    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSourceName()
    */
    @Override
    public String getSourceName() {
        return methodName;
    }

    /*
    *(non-Javadoc)
    *     @see info.silin.an10na.observer.IObserver#getSource()
    */
    @Override
    public Object getSource() {
        return subject;
    }

    /*
    * (non-Javadoc)
    *
    * @see java.lang.Object#hashCode()
    */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        // result = prime * result + ((methodReturnClass == null) ? 0 : methodReturnClass
        // .hashCode());
        result = (prime * result)
                + ((methodName == null) ? 0 : methodName.hashCode());
        result = (prime * result) + ((subject == null) ? 0 : subject.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        @SuppressWarnings("unchecked") final
        MethodObserver<T> other = (MethodObserver<T>) obj;
        if (methodName == null) {
            if (other.methodName != null)
                return false;
        } else if (!methodName.equals(other.methodName)) {
            return false;
        }
        if (subject == null) {
            if (other.subject != null) {
                return false;
            }
        } else if (!subject.equals(other.subject)) {
            return false;
        }
        return true;
    }
}
