/**
 *
 */
package info.silin.an10na.setter;

import static org.fest.reflect.core.Reflection.field;

/**
 * Sets a value of a field directly.
 *
 * @author kostja
 */
public class FieldSetter<T> implements ISetter<T> {

    private final Object target;
    private final String fieldName;
    private final Class<T> targetClass;

    public FieldSetter(final Object target, final String name, final Class<T> targetClass) {
        this.target = target;
        this.fieldName = name;
        this.targetClass = targetClass;
    }

    @Override
    public void set(final T value) {
        field(fieldName).ofType(targetClass).in(target).set(value);
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        // result = prime * result + ((methodReturnClass == null) ? 0 : methodReturnClass.hashCode());
        result = (prime * result)
                + ((fieldName == null) ? 0 : fieldName.hashCode());
        result = (prime * result) + ((target == null) ? 0 : target.hashCode());
        return result;
    }

    /**
     * FieldSetters are considered equaal if the targets and the targetName are equal. TargetClass
     * is a technical detail. Note that if the target's equls is overridden, equality might be
     * asserted for FieldSetter with different target objects
     */
    @Override
    public boolean equals(final Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        @SuppressWarnings("unchecked") final
        FieldSetter<T> other = (FieldSetter<T>) obj;
        if (fieldName == null) {
            if (other.fieldName != null)
                return false;
        } else if (!fieldName.equals(other.fieldName))
            return false;
        if (target == null) {
            if (other.target != null)
                return false;
        } else if (!target.equals(other.target))
            return false;
        return true;
    }

    /**
     * Retrieves the name of the targetet field
     * @return the targeted field name
     */
    @Override
    public String getTargetName() {
        return fieldName;
    }

    @Override
    public Object getTarget() {
        return target;
    }
}
