/**
 *
 */
package info.silin.an10na.setter;

import static org.fest.reflect.core.Reflection.method;

/**
 * Passes a value to a method.
 *
 * @author kostja
 */
public class MethodSetter<T> implements ISetter<T> {

    private final Object target;
    private final String methodName;
    private final Class<T> targetClass;

    public MethodSetter(final Object target, final String methodName, final Class<T> methodParamClass) {
        this.target = target;
        this.methodName = methodName;
        this.targetClass = methodParamClass;
    }

    @Override
    public void set(final T value) {
        method(methodName).withParameterTypes(targetClass).in(target).invoke(value);
    }

    @Override
    public Object getTarget() {
        return target;
    }

    @Override
    public String getTargetName() {
        return methodName;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        // result = prime * result + ((methodReturnClass == null) ? 0 : methodReturnClass.hashCode());
        result = (prime * result)
                + ((methodName == null) ? 0 : methodName.hashCode());
        result = (prime * result) + ((target == null) ? 0 : target.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        @SuppressWarnings("unchecked") final
        MethodSetter<T> other = (MethodSetter<T>) obj;
        if (methodName == null) {
            if (other.methodName != null)
                return false;
        } else if (!methodName.equals(other.methodName)) {
            return false;
        }
        if (target == null) {
            if (other.target != null) {
                return false;
            }
        } else if (!target.equals(other.target)) {
            return false;
        }
        return true;
    }
}
